----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:42:38 05/16/2012 
-- Design Name: 
-- Module Name:    filterbank - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

-- Description of entity

-- addr_start 			- given by the input buffer points to the first address in memory to start reading from
-- done 					- triggers when a buffer is ready to be read
-- mem_in_addr 		-	The address where we want to read from in memory
-- data_in				- 	The line that will pass the data from memory
-- data_in_ready		-	Flag from the RAM to tell us the data is on the data_in line
-- mem_out_addr		-	The address to write to in the output RAM
-- data_out				- 	The line that sends the data to the output RAM
-- output_buffer_done 	-  Flag used to inform the output buffer we have sent data
entity filterbank is
    Port ( clk : in STD_LOGIC;
			addr_start : in  STD_LOGIC_VECTOR (7 downto 0);
           done : in  STD_LOGIC;
           mem_in_addr : out  STD_LOGIC_VECTOR (7 downto 0);
           data_in : in  STD_LOGIC_VECTOR (7 downto 0);
			  data_in_ready : in STD_LOGIC;
           mem_out_addr : out  STD_LOGIC_VECTOR (9 downto 0);
           data_out : out  STD_LOGIC_VECTOR (7 downto 0);
			  start_addr_out : out STD_Logic_Vector (9 downto 0);
			  output_buffer_done : out STD_LOGIC);
			  
end filterbank;

architecture Behavioral of filterbank is


-- Signals put the data in to the filter
signal to_filter : std_logic_vector (7 downto 0);

-- Signals to hold the outputs of the filters
signal f_out1: std_logic_vector (7 downto 0);
signal f_out2: std_logic_vector (7 downto 0);
signal f_out3: std_logic_vector (7 downto 0);
signal f_out4: std_logic_vector (7 downto 0);

-- Memory input counter - hold the address in the input buffer we are reading
signal mem_in_counter : std_logic_vector (7 downto 0) := (others => '0');

-- Memory output buffer counter
signal mem_out_counter : std_logic_vector (9 downto 0):= (others => '0');

-- Clock for the filter
signal filter_clk_enable : std_logic := '0';

-- Reset for the filters
signal reset : std_logic := '0';

-- Clock enables for the filters
signal clk_en_filt1 : std_logic := '1';
signal clk_en_filt2 : std_logic := '1';
signal clk_en_filt3 : std_logic := '1';
signal clk_en_filt4 : std_logic := '1';

-- Signal to count up as the signals from the filters come in
signal samples_in : std_logic_vector (2 downto 0) :=  (others=> '0');

--Flag stating buffer is full
signal fDone: STD_LOGIC := '0';

-----------------------------------------------------------------------------------
-----------------------------------------
-- Filter Components
-----------------------------------------
Component Hbp1
    Port ( clk                             :   IN    std_logic; 
         clk_enable                      :   IN    std_logic; 
         reset                           :   IN    std_logic; 
         filter_in                       :   IN    std_logic_vector(7 DOWNTO 0); -- sfix8
         filter_out                      :   OUT   std_logic_vector(7 DOWNTO 0)  -- sfix8
			);
end Component;

Component Hbp2
    Port ( clk                             :   IN    std_logic; 
         clk_enable                      :   IN    std_logic; 
         reset                           :   IN    std_logic; 
         filter_in                       :   IN    std_logic_vector(7 DOWNTO 0); -- sfix8
         filter_out                      :   OUT   std_logic_vector(7 DOWNTO 0)  -- sfix8
			);
end Component;

Component Hbp3
    Port ( clk                             :   IN    std_logic; 
         clk_enable                      :   IN    std_logic; 
         reset                           :   IN    std_logic; 
         filter_in                       :   IN    std_logic_vector(7 DOWNTO 0); -- sfix8
         filter_out                      :   OUT   std_logic_vector(7 DOWNTO 0)  -- sfix8
			);
end Component;

Component Hbp4
    Port ( clk                             :   IN    std_logic; 
         clk_enable                      :   IN    std_logic; 
         reset                           :   IN    std_logic; 
         filter_in                       :   IN    std_logic_vector(7 DOWNTO 0); -- sfix8
         filter_out                      :   OUT   std_logic_vector(7 DOWNTO 0)  -- sfix8
			);
end Component;

begin


	
	-- FIR filter 1
   f1: Hbp1 PORT MAP (
          clk => clk,
			 clk_enable => clk_en_filt1,
          reset => reset,
          filter_in => to_filter,
			 filter_out => f_out1
        );	

	-- FIR filter 2
   f2: Hbp2 PORT MAP (
          clk => clk,
			 clk_enable => clk_en_filt2,
          reset => reset,
          filter_in => to_filter,
			 filter_out => f_out2
        );	

	-- FIR filter 3
   f3: Hbp3 PORT MAP (
          clk => clk,
			 clk_enable => clk_en_filt3,
          reset => reset,
          filter_in => to_filter,
			 filter_out => f_out3
        );	
		  
		  	-- FIR filter 4
   f4: Hbp4 PORT MAP (
          clk => clk,
			 clk_enable => clk_en_filt4,
          reset => reset,
          filter_in => to_filter,
			 filter_out => f_out4
        );	

	-- When the memory is ready with the next piece of data
	-- Get the data and send it to the filters
	process (clk, data_in_ready, done)
		begin
		if(data_in_ready'event and data_in_ready='1') then
			-- Get the data from the memory 
			to_filter <= data_in;
			filter_clk_enable <= '1';
			
			mem_in_counter <= mem_in_counter + '1';
			-- Set the address in ram
			
		end if;
		
		if(done='1' and done'event) then
			
			mem_in_counter <= addr_start;		-- Set the address in ram
			
		end if;
		mem_in_addr <= mem_in_counter;
		
		--mem_in_addr <= mem_in_counter;
	end process;
	
	
	-- Write the out put of the filters to the output buffer
	process(clk, f_out1, f_out2, f_out3, f_out4)
		
		begin
			if (clk'event and clk='1') then
			
			--If the done flag is set, reset it as next stage uses it on sensitivity list
			if (fDone = '1') then
				fDone <= '0';
			end if;
			
			--If address = last address of buffer1 - notify next stage (Ross)
			if (mem_out_counter = conv_std_logic_vector(511, 10)) then
				fDone <= '1';
				start_addr_out <= "0000000000";
			end if;
			
			--If address = last address of buffer2 - notify next stage (Ross)
			if (mem_out_counter = conv_std_logic_vector(1023, 10)) then
				fDone <= '1';
				start_addr_out <= "1000000000";
			end if;
		end if;
			
			-- Update the output ready buffer
			output_buffer_done <= fDone;
			
--		-- If we reach the end of the first half of the buffer -> notify Steve
--		if (mem_out_counter = conv_std_logic_vector(511, 10)) then
--			output_buffer_done<= '1';
--			start_addr_out <= "0000000000";
--			
--				
--		end if;
--			
--				if (mem_out_counter = conv_std_logic_vector(1023, 10)) then
--			output_buffer_done<= '1';
--			start_addr_out <= "1000000000";
--			end if;
--			
--		end if;
--			
--		-- If we reach the end of second half of the buffer -> notify Steve again
--		if (mem_out_counter = conv_std_logic_vector(255, 8)) then
--			output_buffer_done <= '1';
--		else 
--			output_buffer_done<= '0';
--		end if;
			
		-- Put output of filter one on the data line
		if ( f_out1'event) then
			data_out <= f_out1;
			-- Set the memory address to 1*mem_out address
			mem_out_addr	<=  mem_out_counter;
			clk_en_filt1 <= '0'; -- Disable the filter while we wait for the samples from other filters
			samples_in <= samples_in + '1';
				
				-- Put output of filter one on the data line
		elsif ( f_out2'event) then
			data_out <= f_out2;
			-- Set the memory address to 1*mem_out address
			mem_out_addr	<= std_logic_vector(unsigned(mem_out_counter + b"0010000000"));
			clk_en_filt2 <= '0'; -- Disable the filter while we wait for the samples from other filters
			samples_in <= samples_in + '1';
				
		elsif ( f_out3'event) then
				-- Put output of filter one on the data line
			data_out <= f_out3;
			-- Set the memory address to 1*mem_out address
			mem_out_addr	<= std_logic_vector(unsigned(mem_out_counter + b"0100000000"));
			clk_en_filt3 <= '0'; -- Disable the filter while we wait for the samples from other filters
			samples_in <= samples_in + '1';
				
		elsif ( f_out4'event) then
				-- Put output of filter one on the data line
		data_out <= f_out4;
		-- Set the memory address to 1*mem_out address
		mem_out_addr	<= std_logic_vector(unsigned(mem_out_counter + b"0110000000"));
		clk_en_filt4 <= '0'; -- Disable the filter while we wait for the samples from other filters
		samples_in <= samples_in + '1';
		end if;
		
		-- Check if we have all the samples in 
		if (samples_in= "100") then
		samples_in <= "000";
		clk_en_filt1 <= '1';
		clk_en_filt2 <= '1';
		clk_en_filt3 <= '1';
		clk_en_filt4 <= '1';
		
		mem_out_counter <= mem_out_counter + '1';
				
		end if;
		
		-- Increment the memory address

				
		-- Wrap the mem address (If we get to 128 then increase to 512)		
		if (mem_out_counter = b"0010000000") then
				mem_out_counter <= b"1000000000";
		end if;
		
		-- Wrap the mem address (If we get to 784 then reset to 0)
		if (mem_out_counter = b"1010000000") then
				mem_out_counter <= b"0000000000";
		end if;
	
		end process;
		
end Behavioral;

