----------------------------------------------------------------------------------
-- Company: 
-- Engineer: Lloyd Hughes
-- 
-- Create Date:    08:56:55 05/01/2012 
-- Design Name: 
-- Module Name:    sampleBuffer - Behavioral 
-- Project Name: PADAWAN
-- Target Devices: Nexys3
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
USE ieee.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;

--The Cyclic/FIFO input buffer
--Takes in 1 sample is read in at a time and added to a buffer in memory
entity sBuffer is
		Port( 
			en : IN STD_LOGIC;									--Chip/buffer enable
			input_data : IN  STD_LOGIC_VECTOR(7 downto 0);		--Input sample
         clk : IN  std_logic;
         sampleOut : OUT  STD_LOGIC_VECTOR(7 downto 0);--Current output sample (in sample delayed by 1)
         addrOut : OUT  STD_LOGIC_VECTOR(7 downto 0);	 --Address in RAM to store output
         addrStart : OUT  STD_LOGIC_VECTOR(7 downto 0);--Starting address of filled buffer, (6ms delayed sample)
         done : OUT  std_logic								--Flag stating buffer can be read out of memory
			);

end sBuffer;

architecture Behavioral of sBuffer is 
--Tempory address to write next sample to
signal tmpAddr: STD_LOGIC_VECTOR(7 downto 0) := x"00";
--Flag stating buffer is full
signal fDone: STD_LOGIC := '0';
begin

	process(clk)
	begin
		--Process incoming audio on clock cycle when chip is enabled
		if (clk = '1' and clk'event and en = '1') then
			
			--If address = last address of buffer1 - notify next stage (Ross)
			if (tmpAddr = 127) then
				fDone <= '1';
				addrStart <= conv_std_logic_vector(0,8);
			end if;
			
			--If address = last address of buffer2 - notify next stage (Ross)
			if (tmpAddr = 255) then
				fDone <= '1';
				addrStart <= conv_std_logic_vector(128,8);
			end if;
			
			--If the done flag is set, reset it as next stage uses it on sensitivity list
			if (fDone = '1') then
				fDone <= '0';
			end if;
			
			--Increment next address
			tmpAddr <= tmpAddr + '1';
			--Place sample on output buffer
			sampleOut <= input_data;
			--Place output address on buffer
			addrOut <= tmpAddr;
		end if;
	end process;

			--Set done flag
			done <= fDone;

end Behavioral;
 